import logging

import requests
from requests.compat import urljoin  # type: ignore

from typeguard import typechecked

from . import _client, exceptions

logger = logging.getLogger(__name__)


def _check_status_code(response: requests.Response):
    if response.status_code != 200:
        raise exceptions.ServiceNowRequestError(
            f"Received status code {response.status_code} while generating access token. "
            "Please view the logs for more information."
        )


def _extract_access_token(response: requests.Response) -> str:
    try:
        response_body = _client._decode_response(response)
    except exceptions.ServiceNowRequestError:
        raise exceptions.ServiceNowRequestError(
            "Unable to obtain access token. Invalid JSON response. Please view the logs for more information."
        )
    try:
        return response_body["access_token"]
    except KeyError:
        raise exceptions.ServiceNowRequestError(f"Access token missing from the following response: {response_body}")


@typechecked
def from_refresh_token(address: str, *, client_id: str, client_secret: str, refresh_token: str) -> str:
    logger.info("getting new access token using api key authentication")
    response = requests.post(
        urljoin(_client.service_now_base_url(address), "/oauth_token.do"),
        headers={"Content-Type": "application/x-www-form-urlencoded"},
        data={
            "client_id": client_id,
            "client_secret": client_secret,
            "refresh_token": refresh_token,
            "grant_type": "refresh_token",
        },
    )
    _check_status_code(response)
    return _extract_access_token(response)


@typechecked
def from_password_auth(address: str, *, client_id: str, client_secret: str, username: str, password: str) -> str:
    logger.info("getting new access token using password authentication")
    response = requests.post(
        urljoin(_client.service_now_base_url(address), "/oauth_token.do"),
        headers={"Content-Type": "application/x-www-form-urlencoded"},
        data={
            "client_id": client_id,
            "client_secret": client_secret,
            "username": username,
            "password": password,
            "grant_type": "password",
        },
    )
    _check_status_code(response)
    return _extract_access_token(response)
