from typing import List


def html_table(header: List[str], rows: List[List[str]]) -> str:
    """ Generate an HTML table for use in ServiceNow comments.

    :param header: A list of table columns.
    :param rows: A list of lists where each inner list is a table row. See below for an example.

    :returns: An HTML string ready to be used as a ServiceNow record comment.

    Example:

        .. code-block:: python

            html_table(
                ["Name", "Age"],
                [
                    ["Rob", "32"],
                    ["Gary", "24"],
                    ["Bob", "44"]
                ]
            )
    """
    if not isinstance(header, list):
        raise TypeError("Header must be a list.")
    if not any(isinstance(h, str) for h in header):
        raise TypeError("Header elements must be strings.")
    if not isinstance(rows, list):
        raise TypeError("Rows must be a list.")
    if not any(isinstance(r, list) for r in rows):
        raise TypeError("Row elements must be lists.")

    header_style = "padding: 5px; font-size: 13px; text-align: center; " "border: solid gray; border-width: thin;"
    cell_style = "padding: 5px; font-size: 12px; border: solid gray; " "border-width: thin;"

    row_template = """<tr>
{row}
</tr>"""
    table_headers = "\n".join([f'<th style="{header_style}">{head}</th>' for head in header])
    table_header = row_template.format(row=table_headers)
    table_rows = "\n".join(
        [row_template.format(row="\n".join([f'<td style="{cell_style}">{cell}</td>' for cell in row])) for row in rows]
    )

    template = f"""<table>
{table_header}
{table_rows}
</table>
"""
    return template
