import logging

from typing import List, Optional

from typeguard import typechecked

from ._client import ServiceNow
from . import _interactions, exceptions
from ._records import Interaction


logger = logging.getLogger(__name__)


@typechecked
def get_all(
    service_now: ServiceNow, *, display_value: Optional[str] = "false", attributes: Optional[list] = None
) -> List[Interaction]:
    """
    Get all ServiceNow Interactions.

    :param service_now: A ServiceNow Adaptor
    :param attributes: A list of attributes to return in each Interaction Record.
    :param display_value: Return field display values (true), actual values (false), or both (all).

    :raises exceptions.AuthenticationFailed: if authentication failed using the supplied access token.

    :returns: A list of :ref:`records:Interaction` records.
    """
    return _interactions.get_all(service_now, display_value=display_value, attributes=attributes)


@typechecked
def get_by_sys_id(
    service_now: ServiceNow, sys_id: str, *, display_value: Optional[str] = "false", attributes: Optional[list] = None
) -> Interaction:
    """
    Get a ServiceNow interaction by sys_id.

    :param service_now: A ServiceNow Adaptor
    :param sys_id: The sys id of the ServiceNow interaction.
    :param attributes: A list of attributes to return in the Interaction Record.
    :param display_value: Return field display values (true), actual values (false), or both (all).

    :raises exceptions.AuthenticationFailed: if authentication failed using the supplied access token.
    :raises exceptions.NoSuchObject: if no matching interaction was found.

    :returns: A :ref:`records:Interaction` record.
    """
    logger.info(f"looking for interaction with sys_id {sys_id}")
    with exceptions.raise_on_empty(f"No interaction found with sys_id {sys_id}"):
        return _interactions.by_sys_id(service_now, sys_id, display_value=display_value, attributes=attributes)[0]


@typechecked
def get_by_number(
    service_now: ServiceNow, number: str, *, display_value: Optional[str] = "false", attributes: Optional[list] = None
) -> List[Interaction]:
    """
    Get a ServiceNow interaction with a specific number.

    :param service_now: A ServiceNow Adaptor
    :param number: The interaction number to look for.
    :param attributes: A list of attributes to return in the Interaction Record.
    :param display_value: Return field display values (true), actual values (false), or both (all).

    :raises exceptions.AuthenticationFailed: if authentication failed using the supplied access token.
    :raises exceptions.NoSuchObject: if no matching interaction was found.

    :returns: A list of :ref:`records:Interaction` records.
    """
    logger.info(f"looking for interaction with number {number}")
    with exceptions.raise_on_empty(f"No interaction found with number {number}"):
        results = _interactions.search(service_now, display_value=display_value, attributes=attributes, number=number)
        if not results:
            # Caught by raise_on_empty
            raise IndexError
    return results


@typechecked
def search(
    service_now: ServiceNow,
    *,
    attributes: Optional[list] = None,
    display_value: Optional[str] = "false",
    **search_params,
) -> List[Interaction]:
    """
    Search for Service Now Interactions with multiple properties and values.

    :param service_now: A ServiceNow Adaptor
    :param attributes: A list of attributes to return in the Interaction Record.
    :param display_value: Return field display values (true), actual values (false), or both (all).
    :param search_params: Keyword arguments to use in the search. Each keyword must be a valid Interaction field.

    :raises exceptions.AuthenticationFailed: if authentication failed using the supplied access token.
    :returns: A list of :ref:`records:Interaction` records.
    """
    return _interactions.search(service_now, attributes=attributes, display_value=display_value, **search_params)
