import requests
import logging
import functools
import json
from collections import namedtuple
from requests.compat import urljoin  # type: ignore
from requests.utils import prepend_scheme_if_needed
from typing import List, Callable, Optional
from service_now import exceptions

from typeguard import typechecked

logger = logging.getLogger(__name__)


class ServiceNow(namedtuple("ServiceNow", ["address", "access_token", "skip_cert_validation"])):
    @property
    def base_url(self):
        return service_now_base_url(self.address)


@typechecked
def service_now_base_url(address: str):
    return urljoin(prepend_scheme_if_needed(address, "https"), "/api/now/v2/")


@typechecked
def _set_headers(service_now: ServiceNow):
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer {service_now.access_token}",
    }
    return headers


@typechecked
def _form_url(service_now: ServiceNow, url_path: str):
    if url_path.startswith("/"):
        url_path = url_path[1:]
    return urljoin(service_now.base_url, url_path)


def _log_response_info(response: requests.Response):
    logger.error(
        f"request url: {response.request.url}, "
        f"request method: {response.request.method}, "
        f"response status code: {response.status_code}"
    )


def _decode_response(response: requests.Response) -> dict:
    try:
        return response.json()
    except json.decoder.JSONDecodeError as e:
        response_string = response.text.lower()
        if any([string.lower() in response_string for string in {"i got bored", "hibernating instance"}]):
            raise exceptions.InstanceAsleep("ServiceNow instance is sleeping. Please start the instance and try again.")
        _log_response_info(response)
        logger.error(f"response text: {response.text}")
        logger.exception(e)
        raise exceptions.ServiceNowRequestError(
            "Failed to JSON decode response body. Please view the logs for more information."
        )


def _check_status_code(response, response_json, status_codes):
    if response.status_code not in status_codes:
        _log_response_info(response)
        logger.error(f"response body: {response_json}")
        if response.status_code == 401:
            raise exceptions.AuthenticationFailed(
                (
                    "Authentication failed using the supplied access token. "
                    f"Error: {response_json.get('error', 'N/A')}. "
                    f"Description: {response_json.get('error_description', 'N/A')}. "
                    "Please view the logs for more information."
                )
            )
        elif response.status_code == 404:
            raise exceptions.NoSuchObject(f"Received status code 404 from request to url {response.request.url}.")
        else:
            valid_status_codes = ", ".join((str(s) for s in status_codes))
            raise exceptions.ServiceNowRequestError(
                f"Received unexpected status code {response.status_code} from {response.request.method} "
                f"request to {response.request.url}. Valid status codes are {valid_status_codes}. Please view "
                "the logs for more information."
            )


def request(status_codes: tuple = (200,)):
    def _request_wrapper(request_func) -> Callable:
        @functools.wraps(request_func)
        def _inner(service_now, *args, **kwargs):
            response = request_func(service_now, *args, **kwargs)
            response_json = _decode_response(response)
            _check_status_code(response, response_json, status_codes)
            return response_json

        return _inner

    return _request_wrapper


@typechecked
@request()
def get(service_now: ServiceNow, url_path: str, params: Optional[dict] = None, attributes: Optional[List[str]] = None):
    if attributes:
        if params:
            params.update({"sysparm_fields": ",".join(attributes)})
        else:
            params = {"sysparm_fields": ",".join(attributes)}
    url = _form_url(service_now, url_path)
    logger.info(f"sending get request to url {url} with params {params}")
    return requests.get(
        url, headers=_set_headers(service_now), params=params, verify=not service_now.skip_cert_validation
    )


@typechecked
@request()
def put(
    service_now: ServiceNow, url_path: str, data: Optional[dict] = None, params: Optional[dict] = None,
):
    url = _form_url(service_now, url_path)
    logger.info(f"sending put request to url {url} with params {params} and data {data}")
    return requests.put(
        url,
        headers=_set_headers(service_now),
        params=params,
        data=json.dumps(data),
        verify=not service_now.skip_cert_validation,
    )


@typechecked
@request(status_codes=(201,))
def post(service_now: ServiceNow, url_path: str, data: Optional[dict] = None, params: Optional[dict] = None):
    url = _form_url(service_now, url_path)
    logger.info(f"sending post request to url {url} with params {params} and data {data}")
    return requests.post(
        url,
        headers=_set_headers(service_now),
        params=params,
        data=json.dumps(data),
        verify=not service_now.skip_cert_validation,
    )
