import requests
import logging

from typeguard import typechecked

from ._client import ServiceNow

requests.packages.urllib3.disable_warnings()

logger = logging.getLogger(__name__)


@typechecked
def initialize(*, address: str, access_token: str = None, skip_cert_validation: bool = False) -> ServiceNow:
    """ Create a Service Now adaptor. This adaptor should be passed in to functions
    from the other modules in this library, such as the `incidents` and `requested_items` modules.

    :param address: The address for the Service Now Instance to connect to.
    :param access_token: The access token used to authenticate API requests. If you do not already
        have an access token, you can generate one using `access_tokens.from_password_auth` or
        `access_tokens.from_refresh_token`.
    :param skip_cert_validation: Optional. Defaults to False. Set to `True` to skip certificate validation.

    :returns: A Service Now Adaptor.


    :raises exceptions.AuthenticationFailed: if authentication failed using the supplied access token.
    :raises exceptions.ServiceNowRequestError: if the request returned an unsuccessful status code.

    Example 1 - Using a pre-configured access token:

    .. code-block:: python

        service_now = client.initialize(
            address='abc123.service-now.com',
            access_token='jhvdhsi7478wqeujfbwevvg6w7f87euoqwbv'
        )

    Example 2 - Using an access token generated by a function in the `access_tokens` module:

    .. code-block:: python

        address = 'abc123.service-now.com'

        access_token = access_tokens.from_refresh_token(
            address,
            client_id="a80vb5768523d1313300b6878bc9",
            client_secret="2Rk0XtVz12l",
            refresh_token="09876rtweqgwfdbwnvefihquwgy78fq
        )

        service_now = client.initialize(
            address=address,
            access_token=access_token
        )


    """
    return ServiceNow(address, access_token, skip_cert_validation)
