import logging
from typing import List, Optional
from typeguard import typechecked
from service_now._client import ServiceNow
from service_now._records import User
from service_now import _users, exceptions

logger = logging.getLogger(__name__)


@typechecked
def get_all(service_now: ServiceNow, *, attributes: Optional[list] = None) -> List[User]:
    """
    Get all ServiceNow system users.

    :param service_now: A ServiceNow Adaptor
    :param attributes: A list of attributes to return in each User Record.

    :raises exceptions.AuthenticationFailed: if authentication failed using the supplied access token.

    :returns: A list of :ref:`records:User` records.
    """
    return _users.get_all(service_now, attributes)


@typechecked
def get_by_sys_id(service_now: ServiceNow, sys_id: str, *, attributes: Optional[list] = None) -> User:
    """
    Get a ServiceNow system user by sys_id.

    :param service_now: A ServiceNow Adaptor
    :param sys_id: The sys id of the ServiceNow system user.
    :param attributes: A list of attributes to return in the User Record.

    :raises exceptions.AuthenticationFailed: if authentication failed using the supplied access token.
    :raises exceptions.NoSuchObject: if no matching User was found.

    :returns: A :ref:`records:User` record.
    """
    logger.info(f"looking for system user with sys_id {sys_id}")
    with exceptions.raise_on_empty(f"No user found with sys_id {sys_id}"):
        return _users.by_sys_id(service_now, sys_id, attributes=attributes)[0]


@typechecked
def get_by_username(service_now: ServiceNow, username: str, *, attributes: Optional[list] = None) -> User:
    """
    Get a ServiceNow system user by name.

    :param service_now: A ServiceNow Adaptor
    :param username: The username of the ServiceNow system user.
    :param attributes: A list of attributes to return in the User Record.

    :raises exceptions.AuthenticationFailed: if authentication failed using the supplied access token.
    :raises exceptions.NoSuchObject: if no matching User was found.

    :returns: A :ref:`records:User` record.
    """
    logger.info(f"looking for system user with user_name {username}")
    with exceptions.raise_on_empty(f"No user found with user_name {username}"):
        return _users.search(service_now, attributes=attributes, user_name=username)[0]


@typechecked
def search(service_now: ServiceNow, *, attributes: Optional[list] = None, **search_params) -> List[User]:
    """
    Search for Service Now Users with multiple properties and values.

    :param service_now: A ServiceNow Adaptor
    :param search_params: Keyword arguments to use in the search. Each keyword must be a valid User field.
    :param attributes: A list of attributes to return in each User Record.

    :raises exceptions.AuthenticationFailed: if authentication failed using the supplied access token.
    :raises exceptions.NoSuchObject: if no matching User was found.

    :returns: A list of :ref:`records:User` records.
    """
    return _users.search(service_now, attributes=attributes, **search_params)


@typechecked
def update(service_now: ServiceNow, sys_id: str, **update_params) -> User:
    """
    Update a ServiceNow User.

    :param service_now: A ServiceNow Adaptor.
    :param update_params: Keyword arguments to use when updating a User. Each keyword must be a valid User field.

    :raises exceptions.NoSuchObject: if no updated User record was returned.

    :returns: A :ref:`records:User` record.
    """
    logger.info(f"updating user with sys_id {sys_id} and parameters {update_params}")
    with exceptions.raise_on_empty("No updated user was returned. Please view the logs for more information."):
        return _users.update(service_now, sys_id, **update_params)[0]


@typechecked
def create(service_now: ServiceNow, **user_data) -> User:
    """
    Create a ServiceNow user.

    :param service_now: A ServiceNow Adaptor.
    :param user_data: Keyword arguments to use when updating a User. Each keyword must be a valid User field.

    :raises exceptions.NoSuchObject: if no updated User record was returned.

    :returns: A :ref:`records:User` record.
    """
    logger.info(f"creating user with parameters {user_data}")
    with exceptions.raise_on_empty("No new user was returned. Please view the logs for more information."):
        return _users.create(service_now, **user_data)[0]
