"""Access Tokens

Use the actions here to create an [AccessDetails Dictionary](../inputs_outputs/#accessdetails) if you don't have an access token.
"""
from service_now import access_tokens

from .records import AccessDetails


def from_refresh_token(address: str, client_id: str, client_secret: str, refresh_token: str) -> AccessDetails:
    """Generate an OAuth access token using a refresh token.

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"

    - __address:__ the ServiceNow instance address

    - __client_id:__ the client_id for the PPA OAuth application

    - __client_secret:__ the client_secret for the PPA OAuth application

    - __refresh_token:__ a refresh token generated for the PPA OAuth application

??? success "Output"
    An [AccessDetails Dictionary](../inputs_outputs/#AccessDetails).

??? example
    - In this example the sensitive information was retrieved earlier from a vault (not shown)

    ```YAML linenums="1"
    - service_now.access_tokens.from_refresh_token:
        address: my-instance.servicenow.com
      load:
        client_id: service_now_client_id
        client_secret: service_now_client_secret
        refresh_token: service_now_refresh
      save: access_details
    ```
    """
    return AccessDetails(
        address=address,
        access_token=access_tokens.from_refresh_token(
            address, client_id=client_id, client_secret=client_secret, refresh_token=refresh_token
        ),
    )


def from_password_auth(address: str, client_id: str, client_secret: str, username: str, password: str) -> AccessDetails:
    """Generate an OAuth access token using a username & password.

The access token will be generated for the identity supplied as username.

!!! question "Minimum Plugin Version: 1.1.0"

??? info "Input"

    - __address:__ the ServiceNow instance address

    - __client_id:__ the client_id for the PPA OAuth application

    - __client_secret:__ the client_secret for the PPA OAuth application

    - __username:__ username for authentication

    - __password:__ password for authentication

??? success "Output"
    An [AccessDetails Dictionary](../inputs_outputs/#AccessDetails).

??? example
    - In this example the sensitive information was retrieved earlier from a vault (not shown)

    ```YAML linenums="1"
    - service_now.access_tokens.from_password_auth:
        address: my-instance.servicenow.com
      load:
        client_id: service_now_client_id
        client_secret: service_now_client_secret
        username: service_now_username
        password: service_now_password
      save: access_details
    ```
    """
    return AccessDetails(
        address=address,
        access_token=access_tokens.from_password_auth(
            address, client_id=client_id, client_secret=client_secret, username=username, password=password
        ),
    )
