"""
Interactions

This module contains actions related to ServiceNow Interactions.

!!! question "Minimum Plugin Version: 6.0.0"
"""
import uuid
import logging

from typing import List, Union, Optional

from service_now import interactions, users, exceptions

from ppa import ui

from ._wrappers import _api_function
from .records import Interaction, InteractionData, AccessDetails


logger = logging.getLogger(__name__)


INTERACTION_TABLE_HEADER = ["Number", "Short Description", "State", "Assigned To", "Active"]

INTERACTION_TABLE_FIELDS = [
    lambda interaction: interaction.get("number", ""),
    lambda interaction: interaction.get("short_description", ""),
    lambda interaction: interaction.get("state", ""),
    lambda interaction: interaction.get("assigned_to", {}).get("name", ""),
    lambda interaction: str(interaction.get("active")),
]


@_api_function
def search(access_details: AccessDetails, **kwargs: Union[str, int, bool]) -> List[Interaction]:
    """Search for interactions. Supports multiple search fields and values.

    Advanced search techniques & filters are not supported yet.

    Each supplied field & value will be converted to _field_=_value_ in the search query.

    !!! question "Minimum Plugin Version: 6.0.0"

    ??? info "Input"

        - __access_details:__ an [AccessDetails Dictionary](../inputs_outputs/#accessdetails)

        - __search_params:__ a dictionary containing [Interaction](../inputs_outputs/#interaction) fields and values

    ??? success "Output"
        A list of [Interaction Dictionaries](../inputs_outputs/#interaction).

    ??? example
        Searching for all _active_ interactions.
        ```YAML linenums="1"
        - service_now.interactions.search:
            active: true
          load:
            access_details: service_now_api_info
          save: active_interactions
        ```"""
    return [
        InteractionData(**interaction._asdict()).__dict__
        for interaction in interactions.search(access_details, display_value="all", **kwargs)
    ]


@_api_function
def get_by_number(access_details: AccessDetails, number: str) -> Interaction:
    """Get an interaction by its number.

    !!! question "Minimum Plugin Version: 6.0.0"

    ??? info "Input"

        - __access_details:__ an [AccessDetails Dictionary](../inputs_outputs/#accessdetails)

        - __number:__ the interaction number to search for

    ??? success "Output"

        A single [Interaction Dictionary](../inputs_outputs/#interaction).

    !!! warning "Interaction Numbers"

        ServiceNow interaction numbers are _not unique_ by default, but for simplicity this action assumes they are.

        If you _do_ have shared interaction numbers, you can use the [search](#search) action to get multiples.

    ??? example
        ```YAML linenums="1"
        - service_now.interactions.get_by_number:
            number: TEST-INTERACTION
          load:
            access_details: service_now_api_info
          save: test_interaction
        ```"""
    return InteractionData(**interactions.get_by_number(access_details, number, display_value="all")[0]._asdict()).__dict__


@_api_function
def get_by_number_interactive(access_details: AccessDetails) -> Interaction:
    """Interactively search for an interaction using the _number_ field.

    !!! question "Minimum Plugin Version: 6.0.0"

    ??? info "Input"

        - __access_details:__ an [AccessDetails Dictionary](../inputs_outputs/#accessdetails)

    ??? success "Output"
        An [Interaction Dictionary](../inputs_outputs/#interaction).

    ??? example
        ```YAML linenums="1"
        - service_now.interactions.get_by_number_interactive:
          load:
            access_details: service_now_api_info
          save: interaction
        ```"""
    header = ["Number", "Short Description", "Opened By", "Opened At"]
    input_id = str(uuid.uuid4())
    error_id = str(uuid.uuid4())
    input_table_id = str(uuid.uuid4())
    output_table_id = str(uuid.uuid4())
    progress_id = str(uuid.uuid4())

    def _search():
        while True:
            number = ui.input_text("Interaction Number", required=True, element_id=input_id)
            ui.output_progress(f"Looking for Interaction {number}...", percent=None, element_id=progress_id)
            try:
                results = interactions.get_by_number(access_details, number, display_value="all")
                if len(results) > 1:
                    ui.output_progress(f"Found {len(results)} Interactions", percent=100, element_id=progress_id)
                else:
                    ui.output_progress(f"Found Interaction {number}", percent=100, element_id=progress_id)
                break
            except exceptions.NoSuchObject:
                ui.output_progress("No Interaction found", percent=0, element_id=progress_id)
                ui.output_error(f"No Interaction found with number {number}. Please try again.", element_id=error_id)
            except Exception as e:
                ui.output_progress("Operation failed", percent=0, element_id=progress_id)
                raise e
        return results

    def _select(results):
        if len(results) == 1:
            return next(iter(results))
        table_data = []
        for interaction in results:
            opener = users.get_by_sys_id(access_details, interaction.opened_by.value)
            opener_name = getattr(opener, "name", getattr(opener, "user_name"))
            table_data.append(
                [
                    interaction.number.display_value,
                    interaction.short_description.display_value,
                    opener_name,
                    interaction.opened_at.display_value,
                ]
            )
        while True:
            indices = ui.input_table("Please Select an Interaction", header=header, data=table_data, element_id=input_table_id)
            if any([not indices, len(indices) > 1]):
                ui.output_error("Please select one interaction.", element_id=error_id)
                continue
            return next(iter((results[i] for i in indices)))

    def _confirm(interaction):
        opener = users.get_by_sys_id(access_details, interaction.opened_by.value)
        opener_name = getattr(opener, "name", getattr(opener, "user_name"))
        ui.output_table(
            "Interaction Details",
            header=header,
            data=[
                [
                    interaction.number.display_value,
                    interaction.short_description.display_value,
                    opener_name,
                    interaction.opened_at.display_value,
                ]
            ],
            element_id=output_table_id,
        )
        if ui.input_confirm("Is this the correct Interaction?", confirm="Yes", cancel="No"):
            return True
        return False

    for _ in range(0, 10):
        results = _search()
        chosen_interaction = _select(results)

        if not _confirm(chosen_interaction):
            if not ui.input_confirm("No Interaction selected, search again?", confirm="Yes", cancel="No"):
                raise exceptions.NoSuchObject("No Interaction was selected.")
            ui.output_error("Please search again.")
        else:
            return InteractionData(**chosen_interaction._asdict()).__dict__
    raise Exception("No Interaction selected in 10 attempts. Operation failed.")


def output_table(text: str, interaction_list: List[Interaction]) -> None:
    """Display a list of interactions in a table.

    The table will have the following columns:

    - Number
    - Short Description
    - State
    - Assigned To
    - Active

    !!! question "Minimum Plugin Version: 6.0.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __interaction_list:__ a list of [Interaction Dictionaries](../inputs_outputs/#interaction) to display in the table

    ??? success "Output"

        Nothing is outputted by this action.

    ??? example

        {% raw %}
        ```YAML linenums="1"
        - service_now.users.search:
            user_name: john.smith
          load:
            access_details: service_now_api_info
          save: users

        - ppa_tools.lists.get_first_item:
          load:
            items: users
          save: user

        - service_now.interactions.search:
          load:
            assigned_to: user.sys_id
            access_details: service_now_api_info
          save: user_interactions

        - service_now.interactions.output_table:
            text: Interactions for {{ user.user_name }}
          load:
            interaction_list: user_interactions
        ```
        {% endraw %}
    """
    ui.output_table(
        text,
        header=INTERACTION_TABLE_HEADER,
        data=[[get(interaction) for get in INTERACTION_TABLE_FIELDS] for interaction in interaction_list],
    )


def input_table(
    text: str, interaction_list: List[Interaction], minimum: Optional[int] = None, maximum: Optional[int] = None
) -> List[Interaction]:
    """Display a list of interactions in a table, & allow the task operator to make a selection.

    The table will have the following columns:

    - Number
    - Short Description
    - State
    - Assigned To
    - Active

    !!! question "Minimum Plugin Version: 6.0.0"

    ??? info "Input"

        - __text:__ The title of the table

        - __interaction_list:__ A list of [Interaction Dictionaries](../inputs_outputs/#interaction) to display in the table

        - __minimum:__ The minimum number of acceptable selections

        - __maximum:__ The maximum number of acceptable selections

    ??? success "Output"

        A list of [Interaction Dictionaries](../inputs_outputs/#interaction).

    ??? tip
        - If neither a _minimum_ or _maximum_ is provided, the task operator will be able to submit 0 selections.
        - If _minimum_ or _maximum_ are provided, the operation will repeat until the task operator makes a valid number
            of selections.

    ??? example
        ```YAML linenums="1"
        - service_now.interactions.search:
          load:
            access_details: service_now_api_info
          save: interactions

        - service_now.interactions.input_table:
            text: Please Select 1 Interaction
            minimum: 1
            maximum: 1
          load:
            interaction_list: interactions
          save: interaction
        ```
    """
    table_element_id = str(uuid.uuid4())
    error_element_id = str(uuid.uuid4())
    while True:
        indices = ui.input_table(
            text,
            header=INTERACTION_TABLE_HEADER,
            data=[[get(interaction) for get in INTERACTION_TABLE_FIELDS] for interaction in interaction_list],
            element_id=table_element_id,
        )
        count = len(indices)
        if minimum and count < minimum:
            ui.output_error(f"Please choose at least {minimum} record(s)", element_id=error_element_id)
            continue
        if maximum and count > maximum:
            ui.output_error(f"Please choose no more than {maximum} record(s)", element_id=error_element_id)
            continue
        break

    return [interaction_list[i] for i in indices]
