"""
Requested Items

This module contains actions related to ServiceNow Requested Items.
"""
import uuid

from typing import List, Union

from service_now import requested_items, exceptions

from ppa import ui

from ._wrappers import _api_function
from .records import RequestedItem, RequestedItemData, AccessDetails


@_api_function
def search(access_details: AccessDetails, **search_params: Union[str, int, bool]) -> List[RequestedItem]:
    """Search for requested items. Supports multiple search fields and values.

Advanced search techniques & filters are not supported yet.

Each supplied field & value will be converted to _field_=_value_ in the search query.

!!! question "Minimum Plugin Version: 4.0.0"

??? info "Input"

    - __access_details:__ an [AccessDetails Dictionary](../inputs_outputs/#accessdetails)

    - __search_params:__ a dictionary containing [Requested Item](../inputs_outputs/#requesteditem) fields and values

??? success "Output"
    A list of [RequestedItem Dictionaries](../inputs_outputs/#requesteditem).

??? example
    Searching for all _active_ requested items.
    ```YAML linenums="1"
    - service_now.requested_items.search:
        active: true
      load:
        access_details: service_now_api_info
      save: active_requested_items
    ```
"""
    return [
        RequestedItemData(**requested_item._asdict()).__dict__
        for requested_item in requested_items.search(access_details, *search_params)
    ]


@_api_function
def get_by_number(access_details: AccessDetails, number: str) -> RequestedItem:
    """Get a requested item by its number.

!!! question "Minimum Plugin Version: 4.0.0"

??? info "Input"

    - __access_details:__ an [AccessDetails Dictionary](../inputs_outputs/#accessdetails)

    - __number:__ the requested item number to search for

??? success "Output"

    A single [RequestedItem Dictionary](../inputs_outputs/#requesteditem).

!!! warning "Requested Item Numbers"

    ServiceNow requested item numbers are _not unique_ by default, but for simplicity this action assumes they are.

    If you _do_ have shared requested item numbers, you can use the [search](#search) action to get multiples.

??? example
    ```YAML linenums="1"
    - service_now.requested_items.get_by_number:
        number: TEST-REQUESTED-ITEM
      load:
        access_details: service_now_api_info
      save: test_requested_item
    ```
"""
    return RequestedItemData(**requested_items.get_by_number(access_details, number)._asdict()).__dict__


@_api_function
def update(access_details: AccessDetails, sys_id: str, **update_data: Union[str, int, bool]) -> RequestedItem:
    """Update a requested item using one or more sets of fields & values.

!!! question "Minimum Plugin Version: 4.0.0"

??? info "Input"

    - __access_details:__ an [AccessDetails Dictionary](../inputs_outputs/#accessdetails)

    - __sys_id:__ the _sys_id_ of the requested item to update

    - __update_data:__ a dictionary containing [Requested Item Dictionary](../inputs_outputs/#requesteditem) fields and values

??? success "Output"
    The updated requested item as a [RequestedItem Dictionary](../inputs_outputs/#requesteditem).

??? example
    Resolving requested item _TEST-REQUESTED-ITEM_ with a comment and note.

    ```YAML linenums="1"
    - service_now.requested_items.update:
        number: TEST-REQUESTED-ITEM
        state: 6
        close_notes: Resolved via PPA
        comment: Resolving a requested item from a PPA task
      load:
        access_details: service_now_api_info
        sys_id: requested_item.sys_id
      save: resolved_requested_item
    ```
"""
    return RequestedItemData(**requested_items.update(access_details, sys_id, **update_data)._asdict()).__dict__


@_api_function
def get_by_number_interactive(access_details: AccessDetails) -> RequestedItem:
    """Interactively search for a requested item using the _number_ field.

!!! question "Minimum Plugin Version: 4.0.0"

??? info "Input"

    - __access_details:__ an [AccessDetails Dictionary](../inputs_outputs/#accessdetails)

??? success "Output"
    A [RequestedItem Dictionary](../inputs_outputs/#requesteditem).

??? example
    ```YAML linenums="1"
    - service_now.requested_items.get_by_number_interactive:
      load:
        access_details: service_now_api_info
      save: requested_item
    ```
"""
    header = ["Number", "Short Description", "Opened At"]
    input_id = str(uuid.uuid4())
    error_id = str(uuid.uuid4())
    output_table_id = str(uuid.uuid4())
    progress_id = str(uuid.uuid4())

    def _search():
        while True:
            number = ui.input_text("Requested Item Number", required=True, element_id=input_id)
            ui.output_progress(f"Looking for Requested Item {number}...", percent=None, element_id=progress_id)
            try:
                requested_item = requested_items.get_by_number(access_details, number)
                ui.output_progress(f"Found Requested Item {number}", percent=100, element_id=progress_id)
                break
            except exceptions.NoSuchObject:
                ui.output_progress("No Requested Item found", percent=0, element_id=progress_id)
                ui.output_error(f"No Requested Item found with number {number}. Please try again.", element_id=error_id)
            except Exception as e:
                ui.output_progress("Operation failed", percent=0, element_id=progress_id)
                raise e
        return requested_item

    def _confirm(requested_item):
        ui.output_table(
            "Requested Item Details",
            header=header,
            data=[[requested_item.number, requested_item.short_description, requested_item.opened_at]],
            element_id=output_table_id,
        )
        if ui.input_confirm("Is this the correct Requested Item?", confirm="Yes", cancel="No"):
            return True
        return False

    for _ in range(0, 10):
        requested_item = _search()
        if not _confirm(requested_item):
            if not ui.input_confirm("No Requested Item selected, search again?", confirm="Yes", cancel="No"):
                raise exceptions.NoSuchObject("No Requested Item was selected.")
            ui.output_error("Please search again.")
        else:
            return RequestedItemData(**requested_item._asdict()).__dict__
    raise Exception("No Requested Item selected in 10 attempts. Operation failed.")
