"""
Users

This module contains actions related to ServiceNow System Users.
"""
from typing import Union, List


from service_now import users

from ._wrappers import _api_function
from .records import User, UserData, AccessDetails


@_api_function
def search(access_details: AccessDetails, **kwargs: Union[str, int, bool]) -> List[User]:
    """Search for users. Supports multiple search fields and values.

Advanced search techniques & filters are not supported yet.

Each supplied field & value will be converted to _field_=_value_ in the search query.

!!! question "Minimum Plugin Version: 2.0.0"

??? info "Input"

    - __access_details:__ an [AccessDetails Dictionary](../inputs_outputs/#accessdetails)

    - __search_params:__ a dictionary containing [User](../inputs_outputs/#user) fields and values

??? success "Output"

    A list of [User Dictionaries](../inputs_outputs/#user).

??? example
    ```YAML linenums="1"
    - service_now.users.search:
        user_name: John Smith
      load:
        access_details: service_now_api_info
      save: user
    ```
"""
    return [UserData(**user._asdict()).__dict__ for user in users.search(access_details, **kwargs)]


@_api_function
def by_sys_id(access_details: AccessDetails, sys_id: str) -> User:
    """Get a user by _sys_id_.

Commonly used to resolve the _opened_by_, _caller_id_, & _closed_by_ keys in:

- [Incident](../inputs_outputs/#incident) Dictionaries
- [RequestedItem](../inputs_outputs/#requesteditem) Dictionaries

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"

    - __access_details:__ an [AccessDetails Dictionary](../inputs_outputs/#accessdetails)

    - __sys_id:__ the _sys_id_ of the user

??? success "Output"
    A [User Dictionary](../inputs_outputs/#user).

??? example
    ```YAML linenums="1"
    - service_now.users.get_by_sys_id:
      load:
        access_details: service_now_api_info
        sys_id: incident.caller_id
      save: caller
    ```
"""
    return next(iter((UserData(**user._asdict()).__dict__ for user in users.get_by_sys_id(access_details, sys_id))))
